@@ -2,7 +2,7 @@
  *
  * Contains calife's specific defines
  *
- * $Id: //depot/security/calife/main/acconfig.h#3 $
+ * $Id: //depot/security/calife/main/acconfig.h#4 $
  */
 
 /* Define if you have the <string.h> header file.  */
@@ -14,6 +14,9 @@
 /* Define if you have the <sys/param.h> header file.  */
 #undef HAVE_SYS_PARAM_H
 
+/* Define if you have the <libgen.h> header file.  */
+#undef HAVE_LIBGEN_H
+
 /* Define if you have the <syslog.h> header file.  */
 #undef HAVE_SYSLOG_H
 
@@ -29,6 +32,9 @@
 /* Define it if you have SCO's /etc/shadow file */
 #undef HAVE_SCO_ETC_SHADOW
 
+/* Define it if you have an /etc/shadow file */
+#undef HAVE_ETC_SHADOW
+
 /* Define if you don't want to type a password at all */
 #undef RELAXED
 
