@@ -20,7 +20,7 @@
  **
  ** Compilation:    via le makefile
  **
- ** Copyright (c) 1991-1999 by Ollivier ROBERT
+ ** Copyright (c) 1991-2002 by Ollivier ROBERT
  ** A distribuer selon les regles de la GNU GPL General Public Licence
  ** Voir le fichier COPYING fourni.
  **
@@ -162,6 +162,15 @@
  **                 * Correction bug si argv[1] == '-'.
  **                 * Correction bug d'allocation mmoire, trouv par
  **                   Thomas Quinot.
+ **                 * Calife exige un tty associ  stdin, trouv par
+ **                   Laurent Wacrenier.
+ **                 * Compilation sur Hurd + simplification des strcpy.
+ **                   Thomas Quinot.
+ **                 * Utilise utmp aussi sous Linux et amlioration du
+ **                   traitement.
+ **                 * Utilise un code analogue  su(8) pour pallier aux
+ **                   dficiences de getlogin(3).
+ **                 * Contourne un bug de getpass(3) sur Linux/glibc.
  **/
 
 #define MAIN_MODULE
@@ -172,10 +181,10 @@
 #include "conf.h"
 
 #ifndef lint
-static char * rcsid = "$Id: //depot/security/calife/main/calife.c#38 $";
+static char * rcsid = "$Id: //depot/security/calife/main/calife.c#44 $";
 #endif /* lint */
 
-FILE    * fp = NULL, * log = NULL;  /* fichier d'auth. et log */
+FILE    * fp = NULL, * logfile = NULL;  /* fichier d'auth. et log */
 int     custom_shell= 0;            /* modification du shell ? */
 char    * shell;                    /* nom du shell */
 uid_t   ssid;   					/* saved uid -- POSIX */
@@ -199,19 +208,19 @@ main(argc, argv)
     char            * name, * user_to_be, * tty, this_time [30];
     char            * out_rc, * login;
     char            * uargv;
+    uid_t           uid;
     time_t          t;
-    struct passwd * test_user;
+    struct passwd * test_user, *pwd;
 #ifdef HAVE_POSIX_SIGNALS
     sigset_t        old_set, sig_set;
 #endif
 
-    name = NULL;    
+    name = NULL;
+    tty = NULL;
     ssid = geteuid ();  /* get saved uid for swapping -- security reasons */
-#ifdef HPUX
-    setresuid (-1,getuid (),-1);/* stay non root for a time */
-#else
-    seteuid (getuid ());/* stay non root for a time */
-#endif /* HPUX */
+
+    RELEASE_ROOT;
+
     if (ssid != 0)                /* ... not setuid, exit */
         exit (11);
     
@@ -220,12 +229,12 @@ main(argc, argv)
 #ifdef STDC_HEADERS
     fprintf (stderr, "Calife : How to become someone else legally\n"
                      "Distribute in respect of the GNU General Public Licence\n"
-                     "Copyright (c) 1991, 1999 by Ollivier ROBERT\n"
+                     "Copyright (c) 1991, 2002 by Ollivier ROBERT\n"
                      "%s\n", rcsid);
 #else /* !STDC_HEADERS */
     fprintf (stderr, "Calife : How to become someone else legally\n"
     fprintf (stderr, "Distribute in respect of the GNU General Public Licence\n");
-    fprintf (stderr, "Copyright (c) 1991, 1999 by Ollivier ROBERT\n");
+    fprintf (stderr, "Copyright (c) 1991, 2002 by Ollivier ROBERT\n");
     fprintf (stderr, "%s\n", rcsid);
 #endif /* STDC_HEADERS */
     fprintf (stderr, "Options : ");
@@ -258,6 +267,12 @@ main(argc, argv)
     (void) signal (SIGQUIT, SIG_IGN);
     (void) signal (SIGINT, SIG_IGN);
     (void) signal (SIGHUP, SIG_IGN);
+    (void) signal (SIGABRT, SIG_IGN);
+    (void) signal (SIGTERM, SIG_IGN);
+    (void) signal (SIGCHLD, SIG_IGN);
+#ifdef SIGTTOU
+    (void) signal (SIGTTOU, SIG_IGN);
+#endif /* SIGTTOU */
 #else /* HAVE_POSIX_SIGNALS */
     sigemptyset (&sig_set);
     sigaddset (&sig_set, SIGQUIT);
@@ -266,11 +281,19 @@ main(argc, argv)
     sigaddset (&sig_set, SIGABRT);
     sigaddset (&sig_set, SIGTERM);
     sigaddset (&sig_set, SIGCHLD);
+    sigaddset (&sig_set, SIGTTOU);
     sigprocmask (SIG_BLOCK, &sig_set, &old_set);
 #endif
 
+    /*
+     * We insist on having a controlling terminal
+     */
+    if (!isatty (0))
+    {
+        die (1, "No tty associated with stdin, bailing out");
+    }
+
     user_to_be = (char *) xalloc (MAXLOGNAME + 1);
-    tty = (char *) xalloc (MAXPATHLEN);
 
     /*
      * default is root 
@@ -308,7 +331,7 @@ main(argc, argv)
             /*
              * avoid misuse and special characters
              */
-            if (strpbrk (uargv, "-;/!#~@[]{}$*?|<>%\n\t"))
+            if (strpbrk (uargv, " ,\t:+&#%$^()!@~*?<>=|\\/\"\n[]{}"))
                 die (1, "Illegal characters in username `%s'", uargv);
             
             strncpy (user_to_be, uargv, n_len);
@@ -318,13 +341,28 @@ main(argc, argv)
     /*
      * get tty name, for the log file 
      */
-    strcpy (tty, (char *) ttyname (0));
-    strcpy (tty, tty + 5);      /* strip /dev */
+    {
+        char * p = ttyname (0);
+        if (p == NULL)
+            die (1, "No tty associated with stdin, bailing out");
+        tty = (char *) xalloc (strlen (p) - 4);
+        strcpy (tty, p + 5);        /* strip /dev/ */
+    }
 
     /*
      * get login name the standard way
+     *
+     * Code extracted from usr.bin/su/su.c
      */
-    login = (char *) getlogin ();      
+    uid = getuid();
+    login = getlogin();
+    if (login == NULL || (pwd = getpwnam(login)) == NULL ||
+        pwd->pw_uid != uid)
+        pwd = getpwuid(uid);
+    if (pwd == NULL)
+        die (1, "who are you?");
+    login = strdup (pwd->pw_name);
+
     if (login == NULL || *login == '\0')
     {
     /*
@@ -332,7 +370,7 @@ main(argc, argv)
      * entry is invalid thus getlogin(3) returns NULL.
      * We'll look in utmp ourselves.
      */
-#ifdef SUNOS4
+#if defined(SUNOS4) || defined(__linux__)
         int         utmp_fd, get_out = 0, err = 0;
         struct utmp utmp;
 
@@ -363,23 +401,28 @@ main(argc, argv)
          * XXX
          * assuming 8 chars is unwise but there is no other way under SunOS
          * XXX
-         */
-        name = (char *) xalloc (9 * sizeof (char)); 
-        strncpy (name, utmp.ut_name, sizeof utmp.ut_name);
+         */        
         {
+#ifdef UT_NAMESIZE
+            int max_size = UT_NAMESIZE;
+#else /* !UT_NAMESIZE */
+            int max_size = 8;
+#endif /* UT_NAMESIZE */
             int l_name = 0;
             
-            while (name [l_name] && l_name != 8)
+            name = (char *) xalloc ((max_size + 1) * sizeof (char)); 
+            strncpy (name, utmp.ut_name, sizeof utmp.ut_name);
+            while (name [l_name] && l_name != max_size)
                 l_name++;
             name [l_name] = '\0';
         }        
         if (get_out)
         {
-#endif /* !SUNOS4 */
+#endif /* !SUNOS4  && !__linux__ */
             die (1, "You don't exist, get out.");
-#ifdef SUNOS4
+#if defined (SUNOS4) || defined (__linux__)
         }    
-#endif /* SUNOS4 */
+#endif /* SUNOS4 && __linux__ */
     }
     else
     {   
@@ -411,11 +454,11 @@ main(argc, argv)
 #ifndef NO_SYSLOG
         syslog (LOG_AUTH | LOG_ERR, "BAD CALIFE %s to unknown %s on %s", name, user_to_be, tty);
 #else /* NO_SYSLOG */
-        if (log)
+        if (logfile)
         {
-            fprintf (log, "USER=%-9sTTY=%s DATE=%s UNKNOWN NAME=%s <<NO ACCESS>>", name, tty, this_time, user_to_be);
-            fflush (log);
-            fclose (log);
+            fprintf (logfile, "USER=%-9sTTY=%s DATE=%s UNKNOWN NAME=%s <<NO ACCESS>>", name, tty, this_time, user_to_be);
+            fflush (logfile);
+            fclose (logfile);
         }
 #endif /* NO_SYSLOG */
         fprintf (stderr, "Unknown user %s.\n", user_to_be);
@@ -425,7 +468,7 @@ main(argc, argv)
     /*
      * test if user already root 
      */
-    if (getuid () == 0)
+    if (uid == 0)
     {
         if (!strcmp (user_to_be, ROOT_LOGIN))
             die (0, "Already root.");
@@ -482,19 +525,19 @@ main(argc, argv)
                 MESSAGE ("Looking for shell after changing rights...\n");
 
                 p_calife = getpwnam (name); /* return ptr is static */
+                if (p_calife == NULL)
+                    die (1, "Bad pw data errno = %d", errno);
+
                 memcpy (calife, p_calife, sizeof (struct passwd));
 #ifndef NO_SYSLOG
-                if (log)
-                    fclose (log);
+                if (logfile)
+                    fclose (logfile);
 #endif /* !NO_SYSLOG */                
                 {
                     unsigned int e1, e2;
 
-#ifdef HPUX
-                    setresuid (-1,ssid,-1);	/* become root again */
-#else
-                    seteuid (ssid); 		/* become root again */
-#endif
+                    GET_ROOT;
+                    
                     e2 = setgid (wanted_user->pw_gid);
 #ifdef HAVE_INITGROUPS					/* don't know about linux */
                     initgroups (user_to_be, wanted_user->pw_gid);
@@ -541,19 +584,17 @@ main(argc, argv)
                 /*
                  * we write the log file
                  */             
-                if (log)
+                if (logfile)
                 {
-                    fprintf (log, "USER=%-9sTTY=%s DATE=%s NAME=%s\n", name, tty, this_time, user_to_be);
-                    fflush (log);
-                    fclose (log);
+                    fprintf (logfile, "USER=%-9sTTY=%s DATE=%s NAME=%s\n", name, tty, this_time, user_to_be);
+                    fflush (logfile);
+                    fclose (logfile);
                 }
 #endif /* NO_SYSLOG */
                 wait (&status);
-#ifdef HPUX
-                setresuid (-1,getuid (),-1);/* stay non root for a time */
-#else
-                seteuid (getuid ());/* stay non root for a time */
-#endif /* HPUX */
+
+                RELEASE_ROOT;
+
                 out_rc = (char *) xalloc (strlen (CALIFE_OUT_FILE) + 1);
                 strcpy (out_rc, CALIFE_OUT_FILE);
 
@@ -692,12 +733,12 @@ main(argc, argv)
                 name, user_to_be, tty);
         closelog ();
 #else /* NO_SYSLOG */
-        if (log)
+        if (logfile)
         {
-            fprintf (log, "USER=%-9sTTY=%s\tDATE=%s NAME=%s << ACCESS DENIED >>\n",
+            fprintf (logfile, "USER=%-9sTTY=%s\tDATE=%s NAME=%s << ACCESS DENIED >>\n",
                 name, tty, this_time, user_to_be);
-            fflush (log);
-            fclose (log);
+            fflush (logfile);
+            fclose (logfile);
         }
 #endif /* NO_SYSLOG */
         fprintf (stderr, "Calife failed. Sorry.\n");
